/******************************************************************************************/
/*                                                                                        */
/* File Name:  Ovl_Init.asm                                                               */
/*                                                                                        */
/* Date:  August 29, 2003                                                                 */
/*                                                                                        */
/* Purpose:  This file runs through the ROM to look for where the overlays' sections      */
/*           reside.  It then places the info for each section (live address, count       */
/*           size, and type) into respective buffers.  It also checks the types and       */
/*           accounts for how many words to increment in the ROM to look for the          */
/*           next section's info.                                                         */
/*                                                                                        */
/******************************************************************************************/

#include "ovlay.h"
#include <def21364.h>

.extern num_ovl_sec;
.extern total_live_addr;
.extern total_sec_size;
.extern total_sec_type;
.extern read_data_bytes;

.section/dm dm_data;
.var scratch[3];

.section/pm pm_code;
.global _OvlInit;

_OvlInit:
                ustat1 = PPBHC|PPDUR23;
                dm(PPCTL)=ustat1;

                r0=0x1000600;
                dm(EIPP)=r0;
                M12=1;   //DAG2
                dm(EMPP)=m12;
                dm(IMPP)=m12;

                //Init index registers
                I9=total_live_addr;
                I10=total_sec_size;
                I13=total_sec_type;
                I14=num_ovl_sec;

                // ==========================  READ_BOOT_INFO  =======================================
// Places TAG in R0, Internal Count in R2, and Destination Address in R3
// -----------------------------------------------------------------------------------
read_boot_info: CALL read_prom_word;    // read first tag and count
                jump check_routine;

// ==========================  read_prom_word  =======================================
// read_prom_word is a callable subroutine that DMA's in one 48-bit word from PROM.
// It places the MS 32-bits in R3 and the LS 16 (right justified) in R2.
// For example, given the 48-bit word 0x112233445566,
//        R2 holds 0x00005566
//        R3 holds 0x11223344
// -----------------------------------------------------------------------------------
read_prom_word: R0=scratch;
                DM(IIPP)=R0;           // 0x40004 = DMA destination address
                R0=3;
                DM(ICPP)=R0;
                R0=12;
                DM(ECPP)=R0;

                call read_data_bytes;

                R0=dm(scratch);                 // Copy TAG to R0
              RTS (DB);
                    R2=dm(scratch+1);    // Copy count to R2
read_prom_word.end: R3=dm(scratch+2);     // Copy address to R3


// =============================  CHECK_ROUTINE  ==================================
// R0 holds the TAG, R2 holds the Word Count, and R3 holds the Destination Address.
// Overlays have been mapped to a dummy live address in SDRAM space,
// Therefore, check for a destination address of greater than or equal to 0x600000
// in R3.  To advance the count by which to increment readings from the ROM, check
// each tag info and determine whether it needs to be incremented by 12 words, 6
// words, or none.
// Also check for final init.  When final init tag comes across, then the
// end of program is coming and there are no more overlays.
// ---------------------------------------------------------------------------


check_routine:
    R9=PASS R0;             // check TAG

    IF EQ JUMP final_init;  r9=r9-1;    // jump if fetched tag was 0, else tag--
    IF EQ JUMP Init_Zeros;  r9=r9-1;    // jump if fetched tag was 1, else tag--
    IF EQ JUMP Init_Zeros;    r9=r9-1;    // jump if fetched tag was 2, else tag--
    IF EQ JUMP Internal_16;    r9=r9-1;    // jump if fetched tag was 3, else tag--
    IF EQ JUMP Internal_32;    r9=r9-1;    // jump if fetched tag was 4, else tag--
    IF EQ JUMP Internal_48;    r9=r9-1;    // jump if fetched tag was 5, else tag--
    IF EQ JUMP Internal_64;    r9=r9-1;    // jump if fetched tag was 6, else tag--
    IF EQ JUMP initliveinfozeros;   r9=r9-1;    // jump if fetched tag was 7, else tag--
    IF EQ JUMP initliveinfozeros;  r9=r9-1;    // jump if fetched tag was 8, else tag--
    IF EQ JUMP initliveinfo;   r9=r9-1;    // jump if fetched tag was 9, else tag--
    IF EQ JUMP initliveinfo;  jump(pc,0); // jump if fetched tag was A, else it
                                        // was an invalid TAG, so trap for debug

initliveinfo:
    R8 = OVLY_LIVE_START;
    R7 = R3-R8;
    if lt jump Internal_32;
    R8 = OVLY_LIVE_END;
    R7 = R3-R8;
    if gt jump Internal_32;
    r12=dm(EIPP);
    PM(I9,M12) = R12;       //write ROM live address to total_live_addr buffer
    PM(I10,M12) = R2;       //write word count to total_sec_size buffer
    PM(I13,M12) = R0;

    jump Internal_32;

initliveinfozeros:
    R8 = OVLY_LIVE_START;
    R7 = R3-R8;
    if lt jump Init_Zeros;
    R8 = OVLY_LIVE_END;
    R7 = R3-R8;
    if gt jump Init_Zeros;
    PM(I9,M12) = R12;   //write ROM live address to total_live_addr buffer
    PM(I10,M12) = R2;       //write word count to total_sec_size buffer
    PM(I13,M12) = R0;
    jump Init_Zeros;

//no increment needed
Init_Zeros:
    R1=0;   // No data to skip
    R2=0;   // No data to skip
    jump Update_PROM_address;

Internal_16:
    R2=R2+1;    // If count is odd, round up to make it even
    R1=0xFFFFFFFE; // because the loader pads an extra word
    R2=R2 AND R1;  // when the count is odd
    R1=2;   // Count is in 16-bit words, so multiply by 2
    jump Update_PROM_address;

Internal_32:
    R1=4;   // Count is in 32-bit words, so multiply by 4
    jump Update_PROM_address;

Internal_48:
    R2=R2+1;    // If count is odd, round up to make it even
    R1=0xFFFFFFFE; // because the loader pads an extra word
    R2=R2 AND R1;  // when the count is odd
    R1=6;   // Count is in 48-bit words, so multiply by 6
    jump Update_PROM_address;

Internal_64:
    R1=8;   // Count is in 64-bit words, so multiply by 8
    jump Update_PROM_address;

Update_PROM_address:
    R0=dm(EIPP);
    R1=R1*R2(SSI);
    R0=R0+R1;
    DM(EIPP)=R0;
    jump read_boot_info;

final_init:
    rts;
_OvlInit.end: